/**
 * @file  DFFileDevice.cpp
 * @brief Implementierung der Klasse CDFFileDevice.
 */
//////////////////////////////////////////////////////////////////////

#include "../StdAfx.h"
#include "DFFileDevice.h"

CDFFileDevice::CDFFileDevice()
{
	Reset();
}

CDFFileDevice::~CDFFileDevice()
{
}

void CDFFileDevice::Reset()
{
	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	// Member initialisieren
	m_numSerial = 0;

	// Header loeschen
	memset(m_headBuf, 0, sizeof(m_headBuf));
	m_header = (SDFFileDeviceHeader*)m_headBuf;

	// Anwendungsfirmware loeschen
	m_controller.Reset();
	m_isCrypt = 0;
	memset(m_crypt, 0xFF, sizeof(m_crypt));

	// Anzeigedaten loeschen
	m_fileDisplay.Reset();
	// Geraetetexte loeschen
	m_fileDeviceText.Reset();
}


UINT32 CDFFileDevice::GetVersionFile(INT8 *fileName)
{
	// Lesen der Hexdatei
	FILE *fp;
	INT8 data[0x400];
	UINT8 header[DFFILEDEVICE_SIZE_HEADER];
	memset(&header, 0, sizeof(header));

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	UINT32 numLine = 0;
	//UINT32 baseAddress = 0;
	UINT8 count;
	UINT16 i, address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			switch (GetType(data)) {
			case INTELHEX_TYPE_DAT: // Data record
			case INTELHEX_TYPE_EOF: // End-of-file record
			case INTELHEX_TYPE_ESA: // Extended Segment Address Record
			case INTELHEX_TYPE_SSA: // Start Segment Address Record
			case INTELHEX_TYPE_ELA: // Extended Linear Address Record
			case INTELHEX_TYPE_SLA: // Start Linear Address Record
				break;

			case INTELHEX_TYPE_DEVICE_HEADER:

				address = GetAddress(data);
				count   = GetCount(data);
				// spaetere Groessenanderung vorbehalten!
				for (i=address; (i < DFFILEDEVICE_SIZE_HEADER) && ((i - address) < count); i++) {

					header[i] = GetData8(data, i - address);
				}
				break;

			case INTELHEX_TYPE_DEVICE_EEPROM:
			case INTELHEX_TYPE_DEVICE_FLASH:
				break;
			}

			numLine++;
		}
	}
	fclose(fp);	

	return ((SDFFileDeviceHeader*)header)->versionFile;
}

UINT32 CDFFileDevice::GetVersionFirmware(INT8 *fileName, INT8 *versionFirmware)
{
	// Lesen der Hexdatei
	FILE *fp;
	INT8 data[0x400];
	UINT8 headerBuf[DFFILEDEVICE_SIZE_HEADER];
	memset(&headerBuf, 0, sizeof(headerBuf));
	SDFFileDeviceHeader *header = (SDFFileDeviceHeader*)headerBuf;

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	UINT32 numLine = 0;
	//UINT32 baseAddress = 0;
	UINT8 count;
	UINT16 i, address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			switch (GetType(data)) {
			case INTELHEX_TYPE_DAT: // Data record
			case INTELHEX_TYPE_EOF: // End-of-file record
			case INTELHEX_TYPE_ESA: // Extended Segment Address Record
			case INTELHEX_TYPE_SSA: // Start Segment Address Record
			case INTELHEX_TYPE_ELA: // Extended Linear Address Record
			case INTELHEX_TYPE_SLA: // Start Linear Address Record
				break;

			case INTELHEX_TYPE_DEVICE_HEADER:

				address = GetAddress(data);
				count   = GetCount(data);
				// spaetere Groessenanderung vorbehalten!
				for (i=address; (i < DFFILEDEVICE_SIZE_HEADER) && ((i - address) < count); i++) {

					headerBuf[i] = GetData8(data, i - address);
				}
				break;

			case INTELHEX_TYPE_DEVICE_EEPROM:
			case INTELHEX_TYPE_DEVICE_FLASH:
				break;
			}

			numLine++;
		}
	}
	fclose(fp);	

	// Version auswerten (Majornumber muss uebereinstimmen) Erst ab: 03.01.33
	if ((header->versionFile & 0xFFFF0000) != (DFFILEDEVICE_VERSION_AEIV_PZE & 0xFFFF0000)) {

		m_errorState = ES_VERSION_WRONG;
		return FALSE;
	}
	// Abgeaenderter Header ab 03.01.33 ist groeszer 4 Byte
	if (header->sizeData) { // Vorher war nur die Version vorhanden!

		// CRC16 von Header pruefen!
		// Der crcHead ist immer der letzte 16bit Wert in der Struktur und
		// geht ueber den gesamten Dateiheader, sich selbst ausgeschlossen.
		if (*(UINT16*)&headerBuf[header->sizeData - sizeof(UINT16)] != GetCRC16(headerBuf, header->sizeData - sizeof(UINT16))) {

			m_errorState = ES_HEAD_CRC;
			return FALSE;
		}

		// Zu kurz oder zu lang (fehlende Nullterminierung)?
		if ((strlen(header->versionFirmware) < 8) || (strlen(header->versionFirmware) > 32)) {

			m_errorState = ES_FW_VERSION_LENGTH;
			return FALSE;
		}
	} else {

		m_errorState = ES_FUNCTION_UNAVAILABLE;
		return FALSE;
	}

	return sprintf(versionFirmware, "%s", header->versionFirmware);
}

INT32 CDFFileDevice::LoadFirmware(INT8 *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	INT8 data[0x400];
	UINT8 tmpData[0xFF];

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	UINT32 numLine = 0;
	UINT32 baseAddress = 0;
	UINT16 address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			switch (GetType(data)) {
			case INTELHEX_TYPE_DAT: // Data record

				address = GetAddress(data);

				// Adresse darf den maximalen Programmspeicher nicht ueberschreiten
				if ((baseAddress + address + GetCount(data)) > ATMEGA128_SIZE_APPLICATION) {
					
					m_errorState = ES_PROGRAM_OVERFLOW;
					continue;
				}

				GetData(data, tmpData);
				m_controller.SetProgramRange(baseAddress + address, tmpData, GetCount(data));
				break;

			case INTELHEX_TYPE_EOF: // End-of-file record
				break;

			case INTELHEX_TYPE_ESA: // Extended Segment Address Record
				
				baseAddress = GetData16(data, 0) << 4;
				break;

			case INTELHEX_TYPE_SSA: // Start Segment Address Record
				break;

			case INTELHEX_TYPE_ELA: // Extended Linear Address Record
				break;

			case INTELHEX_TYPE_SLA: // Start Linear Address Record
				break;
	
			}

			numLine++;
		}
	}
	fclose(fp);	
			
	return TRUE;
}

INT32 CDFFileDevice::Load(INT8 *fileName) 
{
	// Lesen der Hexdatei
	FILE *fp;
	INT8 data[0x400];
	UINT8 tmpData[0xFF];

	if ((fp = fopen(fileName, "r")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	// Daten eintragen
	INT8  count;
	UINT32 numLine = 0;
	UINT32 baseAddress = 0;
	UINT16 i, address;

	while(!feof(fp) && (m_errorState == ES_NO)) {
	
		if (fgets(data, 0x400, fp) != 0) {
			
			switch (GetType(data)) {
			case INTELHEX_TYPE_DEVICE_HEADER:

				address = GetAddress(data);
				count   = GetCount(data);
				// spaetere Groessenanderung vorbehalten!
				for (i=address; (i < DFFILEDEVICE_SIZE_HEADER) && ((i - address) < count); i++) {

					m_headBuf[i] = GetData8(data, i - address);
				}
				break;

			case INTELHEX_TYPE_DEVICE_EEPROM:
			case INTELHEX_TYPE_DEVICE_FLASH:
				break;

			case INTELHEX_TYPE_DEVICE_CRYPT:

				address = GetAddress(data);
				count   = GetCount(data);

				GetData(data, &m_crypt[address]);
				// Verschluesselt, wenn Verschluesselungsdaten vorhanden sind
				m_isCrypt = 1; 
				break;

			case INTELHEX_TYPE_DAT: // Data record

				address = GetAddress(data);

				// Adresse darf den Maximalen Programmspeicher nicht ueberschreiten
				if ((baseAddress + address + GetCount(data)) > ATMEGA128_SIZE_APPLICATION) {
					
					m_errorState = ES_PROGRAM_OVERFLOW;
					continue;
				}

				GetData(data, tmpData);
				m_controller.SetProgramRange(baseAddress + address, tmpData, GetCount(data));
				break;

			case INTELHEX_TYPE_EOF: // End-of-file record
				break;

			case INTELHEX_TYPE_ESA: // Extended Segment Address Record
				
				baseAddress = GetData16(data, 0) << 4;
				break;

			case INTELHEX_TYPE_SSA: // Start Segment Address Record
				break;

			case INTELHEX_TYPE_ELA: // Extended Linear Address Record
				break;

			case INTELHEX_TYPE_SLA: // Start Linear Address Record
				break;
	
			}

			numLine++;
		}
	}
	fclose(fp);	

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		return FALSE;
	}

	// Version auswerten (Majornumber muss uebereinstimmen) Erst ab: 03.01.33
	if ((m_header->versionFile & 0xFFFF0000) != (DFFILEDEVICE_VERSION_AEIV_PZE & 0xFFFF0000)) {

		m_errorState = ES_VERSION_WRONG;
		return FALSE;
	}
	// Abgeaenderter Header ab 03.01.33 ist groeszer 4 Byte
	if (m_header->sizeData) { // Vorher war nur die Version vorhanden!

		// CRC16 von Header pruefen!
		// Der crcHead ist immer der letzte 16bit Wert in der Struktur und
		// geht ueber den gesamten Dateiheader, sich selbst ausgeschlossen.
		if (*(UINT16*)&m_headBuf[m_header->sizeData - sizeof(UINT16)] != GetCRC16(m_headBuf, m_header->sizeData - sizeof(UINT16))) {

			m_errorState = ES_HEAD_CRC;
			return FALSE;
		}

		// CRC32 von Program-Flash pruefen
		if (m_header->crcProgram != GetCRC32(m_controller.GetMemProgram(0), m_controller.GetProgramLastAddress() - m_controller.GetProgramFirstAddress() + 1)) {

			m_errorState = ES_PROGRAM_CRC;
			return FALSE;
		}
		/*
		switch (m_header.version & 0xFFFF) {
		case (DFFILEDEVICE_VERSION & 0xFFFF):
			break;
		default:
			// Die Minornumber ist ungleich
		}*/
	}

	// Displaydaten laden
	if (!m_fileDisplay.Load(fileName)) {

		m_errorState = ES_LOAD_DISPLAY_DATA;
		return FALSE;
	}
	// Geraetetexte laden
	if (!m_fileDeviceText.Load(fileName)) {

		m_errorState = ES_LOAD_DEVICE_TEXT;
		return FALSE;
	}

	return TRUE;
}

INT32 CDFFileDevice::Save(INT8 *fileName, INT8 *versionFirmware) 
{
	// Schreiben der Hexdatei
	FILE *fp;
	INT8 data[0x400];

	if ((fp = fopen(fileName, "w")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	UINT32 numLine = 0;
	UINT8 lastUsebit = 0, sizeRecord = 0;
	UINT16 i, address = 0;
	UINT32 baseAddress = 0;

	// Version setzen
	m_header->versionFile = DFFILEDEVICE_VERSION_AEIV_PZE;
	m_header->sizeData = sizeof(SDFFileDeviceHeader);

	// Builddatum eintragen
	time_t ltime;
	struct tm *ltm;

	time(&ltime);
	ltm = localtime(&ltime);
	strftime(m_header->timestamp, sizeof(m_header->timestamp), "%Y-%m-%d %H:%M:%S", ltm);

	// Firmwareversion setzen
	if ((strlen(versionFirmware) < 8) || (strlen(versionFirmware) > 32)) {

		fclose(fp);
		m_errorState = ES_FW_VERSION_LENGTH;
		return FALSE;
	}
	strcpy(m_header->versionFirmware, versionFirmware);

	// CRC32 fuer Program-Flash berechnen
	m_header->crcProgram = GetCRC32(m_controller.GetMemProgram(0), m_controller.GetProgramLastAddress() - m_controller.GetProgramFirstAddress() + 1);

	// CRC16 fuer Dateiheader berechnen
	m_header->crcHead = GetCRC16(m_headBuf, sizeof(SDFFileDeviceHeader) - sizeof(UINT16));

	// Dateiheader schreiben
	for (i=0; i < sizeof(SDFFileDeviceHeader); i += INTELHEX_DATA_COUNT) {

		// Extended Segment Address Record
		if (!MakeTypedDataLine(i, INTELHEX_TYPE_DEVICE_HEADER, &m_headBuf[i],
			(sizeof(SDFFileDeviceHeader) - i) > INTELHEX_DATA_COUNT ? INTELHEX_DATA_COUNT : sizeof(SDFFileDeviceHeader) - i, data)) {

			m_errorState = ES_INTELHEX_MAKE_RECORD;
			break;
		}

		fprintf(fp, "%s\n", data);
		numLine++;
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return FALSE;
	}
	
	// Verschluesselungstabelle schreiben
	for (i=0; m_isCrypt && (i < sizeof(m_crypt)); i += INTELHEX_DATA_COUNT) {

		// Extended Segment Address Record
		if (!MakeTypedDataLine(i, INTELHEX_TYPE_DEVICE_CRYPT, &m_crypt[i],
			(sizeof(m_crypt) - i) > INTELHEX_DATA_COUNT ? INTELHEX_DATA_COUNT : sizeof(m_crypt) - i, data)) {

			m_errorState = ES_INTELHEX_MAKE_RECORD;
			break;
		}

		fprintf(fp, "%s\n", data);
		numLine++;
	}

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		fclose(fp);
		return FALSE;
	}

	// Schreiben der Anwendungsfirmware
	for (; (baseAddress + address) < ATMEGA128_SIZE_APPLICATION; address++) {

		if (m_controller.GetProgramUsebit(baseAddress + address)) {

			if (!lastUsebit) {

				if (baseAddress) {
					// write Extended Linear Address Record
					UINT16 add = baseAddress >> 4;
					UINT8 tmp = ((UINT8*)&add)[0];
					((UINT8*)&add)[0] = ((UINT8*)&add)[1];
					((UINT8*)&add)[1] = tmp;

					// Extended Segment Address Record
					if (!MakeTypedDataLine(0, INTELHEX_TYPE_ESA, (UINT8*)&add, 2, data)) {
						
						m_errorState = ES_INTELHEX_MAKE_RECORD;
						break;
					}

					fprintf(fp, "%s\n", data);
					numLine++;
				}

				lastUsebit = 1;
			} 

			sizeRecord++;
		} else if (sizeRecord) {

			// Angefangenen Datenrecord schreiben
			if (!MakeDataLine(address - sizeRecord, m_controller.GetMemProgram(baseAddress + address - sizeRecord), sizeRecord, data)) {

				m_errorState = ES_INTELHEX_MAKE_RECORD;
				break;
			}

			fprintf(fp, "%s\n", data);
			numLine++;

			sizeRecord = 0;

			// Da Datenstrom abbriss, soll die Basisadresse neu gesetzt werden
			lastUsebit = 0;
		} 
		
		if ((sizeRecord == INTELHEX_DATA_COUNT) ||
			(sizeRecord && ((baseAddress + address) == (ATMEGA128_SIZE_APPLICATION - 1)))) {
			// Vollen oder Rest Datenrecord schreiben
			if (!MakeDataLine(address - (sizeRecord - 1), m_controller.GetMemProgram(baseAddress + address - (sizeRecord - 1)), sizeRecord, data)) {

				m_errorState = ES_INTELHEX_MAKE_RECORD;
				break;
			}

			fprintf(fp, "%s\n", data);
			numLine++;

			sizeRecord = 0;
		} 
		
		// Adressbereich ueberschreitet 64k Block
		if (address == 0xFFFF) {

			// evntl. verbliebene Daten rausschreiben
			if (sizeRecord) {

				// Rest Datenrecord schreiben
				if (!MakeDataLine(address - (sizeRecord - 1), m_controller.GetMemProgram(baseAddress + address - (sizeRecord - 1)), sizeRecord, data)) {

					m_errorState = ES_INTELHEX_MAKE_RECORD;
					break;
				}

				fprintf(fp, "%s\n", data);
				numLine++;

				sizeRecord = 0;
			}

			// Basisadresse erhoehen
			baseAddress += address + 1;

			// Da Datenstrom abbriss, soll die Basisadresse neu gesetzt werden
			lastUsebit = 0;
		}
	}
	fclose(fp);

	// Abbruch nach Fehler
	if (m_errorState != ES_NO) {

		return FALSE;
	}

	// Anzeigedaten in Datei schreiben
	if (!m_fileDisplay.Save(fileName, TRUE)) {

		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Geraetetexte
	if (!m_fileDeviceText.Save(fileName, TRUE)) {

		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	// Endezeile eintragen
	if ((fp = fopen(fileName, "a")) == (FILE*)0) {
		// Oeffnen der Datei gescheitert
		m_errorState = ES_FILE_OPEN;
		return FALSE;
	}

	if (!MakeEndLine(data)) {
		
		fclose(fp);
		m_errorState = ES_INTELHEX_MAKE_RECORD;
		return FALSE;
	}

	fprintf(fp, "%s\n", data);
	numLine++;

	fclose(fp);

	return TRUE;
}

INT32 CDFFileDevice::Encrypt(UINT32 numSerial)
{
	UINT32 address;
	UINT32 firstAddress = m_controller.GetProgramFirstAddress();
	UINT32 lastAddress  = m_controller.GetProgramLastAddress();
	UINT8 *mem = m_controller.GetMemProgram(0);

	if (m_isCrypt) {
		// Verschluesselungsveruch obwohl bereits verschluesselt!
		m_errorState = ES_CRYPT_IS;
		return FALSE;
	}

	srand(numSerial);

	for (address=0; address < DFFILEDEVICE_SIZE_CRYPT; address++) {

		m_crypt[address] = (UINT8)(rand() % 0x100);
	}

	m_crypt[DFFILEDEVICE_SN_3] = (UINT8)(numSerial >> 24);
	m_crypt[DFFILEDEVICE_SN_2] = (UINT8)(numSerial >> 16);
	m_crypt[DFFILEDEVICE_SN_1] = (UINT8)(numSerial >> 8);
	m_crypt[DFFILEDEVICE_SN_0] = (UINT8)(numSerial & 0xFF);

	m_crypt[DFFILEDEVICE_CF_3] = (UINT8)(DFFILEDEVICE_CRYPT_FIX >> 24);
	m_crypt[DFFILEDEVICE_CF_2] = (UINT8)(DFFILEDEVICE_CRYPT_FIX >> 16);
	m_crypt[DFFILEDEVICE_CF_1] = (UINT8)(DFFILEDEVICE_CRYPT_FIX >> 8);
	m_crypt[DFFILEDEVICE_CF_0] = (UINT8)(DFFILEDEVICE_CRYPT_FIX & 0xFF);

	for (address=firstAddress; address <= lastAddress; address++) {

		mem[address] ^= m_crypt[address % DFFILEDEVICE_SIZE_CRYPT];
	}
	m_isCrypt = TRUE;

	return TRUE;
}

INT32 CDFFileDevice::Decrypt()
{
	UINT32 cryptFix;
	UINT32 address;
	UINT32 firstAddress = m_controller.GetProgramFirstAddress();
	UINT32 lastAddress  = m_controller.GetProgramLastAddress();
	UINT8 *mem = m_controller.GetMemProgram(0);

	if (!m_isCrypt) {
		// Entschluesselungsversuch ohne vorherige Verschluesselung!
		m_errorState = ES_CRYPT_NONE;
		return FALSE;
	}

	m_numSerial = m_crypt[DFFILEDEVICE_SN_0];
	m_numSerial |= (UINT32)m_crypt[DFFILEDEVICE_SN_1] << 8;
	m_numSerial |= (UINT32)m_crypt[DFFILEDEVICE_SN_2] << 16;
	m_numSerial |= (UINT32)m_crypt[DFFILEDEVICE_SN_3] << 24;

	cryptFix = m_crypt[DFFILEDEVICE_CF_0];
	cryptFix |= (UINT32)m_crypt[DFFILEDEVICE_CF_1] << 8;
	cryptFix |= (UINT32)m_crypt[DFFILEDEVICE_CF_2] << 16;
	cryptFix |= (UINT32)m_crypt[DFFILEDEVICE_CF_3] << 24;

	// Wert ist konstant und dient der Sicherung von der Verschluesselungstabelle
	if (cryptFix != DFFILEDEVICE_CRYPT_FIX) {

		m_errorState = ES_CRYPT_FIXFAIL;
		return FALSE;
	}

	for (address=firstAddress; address <= lastAddress; address++) {

		mem[address] ^= m_crypt[address % DFFILEDEVICE_SIZE_CRYPT];
	}
	m_isCrypt = FALSE;

	return TRUE;
}

INT32 CDFFileDevice::Build(INT8 *fileDevice, INT8 *versionFirmware, INT8 *fileFirmware, INT8 *fileDisplayData, INT8 *fileDeviceText)
{
	// Fehlerobjekt zuruecksetzen
	m_errorState = ES_NO;

	// Laden der Firmware
	if (!LoadFirmware(fileFirmware)) {

		return FALSE;
	}
	// Laden der Anzeigedaten
	if (!m_fileDisplay.Load(fileDisplayData)) {

		return FALSE;
	}
	/*
	// Laden der Geraetetexte
	if (!m_fileDeviceText.Load(fileDeviceText)) {

		return FALSE;
	}
	*/

	// Schreiben der Geraetedatei
	if (!Save(fileDevice, versionFirmware)) {

		return FALSE;
	}

	return TRUE;
}

UINT32 CDFFileDevice::GetSizeData()
{
	return (m_controller.GetProgramLastAddress() - m_controller.GetProgramFirstAddress() + 1)
		+ m_fileDisplay.GetSizeData() + m_fileDeviceText.GetSizeData();
}

