/** 
 * @file   AE3Com.h
 * @author Bernd Ottmann
 * @date   21.02.2003
 * @brief  Deklarationen der Exportroutinen.
 * @note   Die Auflistung der Routinen ist wie in der zugehoerigen Dokumentation.
 */
//////////////////////////////////////////////////////////////////////
#ifndef _AE3COM_H_
#define _AE3COM_H_

/** 
 @mainpage Dokumentation zur DFComDLL.
 <center> @code 

   ########################################################
   ##       |                   |                  |     ##
   ##                          \_/                       ##
   ##       |                 -(_)-                |  \  ##
   ##                          / \                    |\\##
   ##  /|   |                                      |  || ##
   ##/ ||                  _\|||/_                    || ##
   ##  ||   |               /_=_\                  |  || ##
   ##  ||                  ({}-{})                    || ##
   ##  ||   |+-----oOo-------(_)---------------+   |  || ##
   ## //     |                                 |      || ##
   ##/      ||          Datafox GmbH           |   |  || ##
   ##        |                                 |      || ##
   ##       |+-------------------------oOo-----+   |  || ##
   ##                     |  |~|  |                   || ##
   ##       |_ __ _ __ _ _|__| |__|_ __ _ __ _ __ _|  || ##
   ##      /               ||   ||                  \ || ##
   ##    /               .ooO   Ooo.                  \ \##
   ##  /                                                \##
   ########################################################

 @endcode </center>
 @author Bernd Ottmann
 @date   21.02.2003
*/

// ########################################################
// ###################### DEFINES #########################
// ########################################################

/**
 * Version der verwendeten DLL.
 * 
 * @todo Version der verwendeten DLL pruefen, um nur die enthaltenen Routinen zu 
 * importieren oder auch zu exportieren.
 */
// Benutzung ab Version 1.1.i
//#define COM_VERSION          1
//#define COM_VERSION          0x02000000
//#define COM_VERSION          0x02000100
//#define COM_VERSION          0x02000200
//#define COM_VERSION          0x02000300
//#define COM_VERSION          0x02000400
//#define COM_VERSION          0x02000600
#define COM_VERSION          0x02000900

/**
 * Gibt export oder import der Routinen an. Siehe auch EXTERN_DECL.
 *
 * @todo Mit 1 im Quelltext vor #include zu definieren um die Routinen als Import zu erhalten.
 *
 * @code #define AE3COM_EXPORT   1 @endcode
 */
#ifndef AE3COM_EXPORT
	#define AE3COM_EXPORT 0
#endif

#ifndef USE_PRIVATE
	#define USE_PRIVATE   0
#endif

// Eventuell fehlende typedefs ersetzen
#ifndef HANDLE
	#define HANDLE void*
#endif

// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
// +------------+ EXPORTIERTE FUNKTIONEN +----------------+
// ++++++++++++++++++++++++++++++++++++++++++++++++++++++++

/**
 @def EXTERN_DECL Gibt den Praefix fuer Funktionen an. (Dekorierung und Export-, Importangabe)
*/

#if (AE3COM_EXPORT)
	// Einzubindende Bibliotheken: Version.lib, WSock32.lib
	#define EXTERN_DECL         extern "C" __declspec(dllexport)
	// DllGetVersion
	#include "shlwapi.h"
#else
	#define EXTERN_DECL         extern "C" __declspec(dllimport)
#endif


// ########################################################
// ############## ALLGEMEINE FUNKTIONEN ###################
// ########################################################

// ########################################################
/** 
 * @brief Oeffnet die serielle Schnittstelle fuer RS232 / RS485 Direktkommunikation. 
 *
 * Die Funktion ruft intern die Funktion DFCComOpenSerial() mit den Standardparametern
 * iBaudRate = 38400 und iTimeOut = 150 auf.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Kommunikationskanal oeffnen.
 * @retval 1 Schnittstelle erfolgreich geoeffnet.
 * @retval 0 Fehler bei Initialisierung.
 */
EXTERN_DECL int __stdcall DFCComInit(int iComNum);

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Oeffnet die serielle Schnittstelle fuer RS232 / RS485 Direktkommunikation. 
 *
 * Es koennen zusaetzliche Parameter verwendet werden. Die Baudrate ist standardmaessig
 * auf 38400bd einzustellen und der Timeout liegt bei 150ms pro Lesevorgang.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param szComPort Zeiger auf String mit Namen der Schnittstelle "COM1", "COM2" usw..
 * @param iBaudRate Baudrate mit der die Schnittstelle verwendet wird, 38400bd ist Standard.
 * @param iTimeOut Zu verwendender Timeout fuer Read - Befehl. (ReadFile)
 *
 * @return Kommunikationskanal oeffnen.
 * @retval 1 Schnittstelle erfolgreich geoeffnet.
 * @retval 0 Fehler bei Initialisierung.
 */
EXTERN_DECL int __stdcall DFCComOpenSerial(int iComNum, char *szComPort, int iBaudRate, int iTimeOut);

#endif

// ########################################################
/** 
 * @brief Oeffnet die TCP/IP Schnittstelle. 
 *
 * Die Funktion ruft intern die Funktion DFCComOpenSocket() mit den Standardparametern
 * iPortNum = 8000 und iTimeOut = 3000 auf.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param szHost Zeiger auf String mit Hostnamen oder IP - Adresse in Punktnotation.
 *
 * @return Kommunikationskanal oeffnen.
 * @retval 1 Schnittstelle erfolgreich geoeffnet.
 * @retval 0 Fehler bei Initialisierung.
 */
EXTERN_DECL int __stdcall DFCComOpen(int iComNum, char *szHost);

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Oeffnet die TCP/IP Schnittstelle. 
 *
 * Es koennen zusaetzliche Parameter verwendet werden. Die Portnummer ist standardmaessig
 * auf 8000 einzustellen und der Timeout (ein intern verwalteter) 
 * liegt bei 3000 pro Lesevorgang.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param szRemoteIP Zeiger auf String mit Hostnamen oder IP - Adresse in Punktnotation.
 * @param iPortNum Portnummer 
 * @param iTimeOut Zu verwendender Timeout bei Lesevorgang. (keine ms - Einheit)
 *
 * @return Kommunikationskanal oeffnen.
 * @retval 1 Schnittstelle erfolgreich geoeffnet.
 * @retval 0 Fehler bei Initialisierung.
 */
EXTERN_DECL int __stdcall DFCComOpenSocket(int iComNum, char *szRemoteIP, int iPortNum, int iTimeOut);

#endif

// ########################################################
/** 
 * @brief Schliessen des geoeffneten Kanals.
 *
 * Je nach dem Typ des Kanals wird entweder die serielle Schnittstelle oder
 * der Socket geschlossen.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Keine.
 */
EXTERN_DECL void __stdcall DFCComClose(int iComNum);

// ########################################################
/** 
 * @brief Pruefen ob das Geraet erreichbar ist.
 *
 * Ruft intern die Funktion DFCCheckDevice() mit standard - Parametern auf.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 *
 * @return Ergebnis der Erreichbarkeit.
 * @retval 0 Geraet ist nicht erreichbar.
 * @retval 1 Es wurde erfolgreich auf das Geraet zugegriffen.
 */
EXTERN_DECL int __stdcall DFCCheckAE(int iComNum, int iBusNum);

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Pruefen ob das Geraet erreichbar ist.
 *
 * Prueft aufgrund einer kleinen Datenanfrage die Erreichbarkeit des Geraetes.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 * @param iDevicePollRetry Anzahl der Wiederholversuche.
 *
 * @return Ergebnis der Erreichbarkeit.
 * @retval 0 Geraet ist nicht erreichbar.
 * @retval 1 Es wurde erfolgreich auf das Geraet zugegriffen.
 */
EXTERN_DECL int __stdcall DFCCheckDevice(int iComNum, int iBusNum, int *piError, int iDevicePollRetry);

#endif

// ########################################################
/** 
 * @brief Setzt das uebergebene Datum und die Uhrzeit in dem Geraet.
 *
 * Das zuvor aufbereitete Datum mit Uhrzeit muss in einem speziellen Format
 * vorliegen, welches in der Dokumentation zur DLL beschrieben ist.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param pucDateTime Zeiger auf Bytearray mit Werten fuer Datum und Uhrzeit.
 *
 * @return Ergebnis der Wertsetzung im Geraet.
 * @retval 0 Wert konnte nicht gesetz werden.
 * @retval 1 Wert wurde erfolgreich gesetzt.
 */
EXTERN_DECL int __stdcall DFCComSetTime(int iComNum, int iBusNum, unsigned char *pucDateTime);

// ########################################################
/** 
 * @brief Holt das Datum und die Uhrzeit aus dem Geraet.
 *
 * Das Datum mit Uhrzeit wird in einem speziellen Format
 * abgelegt, welches in der Dokumentation zur DLL beschrieben ist.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param pucDateTime Zeiger auf Bytearray mit Werten fuer Datum und Uhrzeit.
 *
 * @return Ergebnis der Wertermittlung aus dem Geraet.
 * @retval 0 Wert konnte nicht ermittelt werden.
 * @retval 1 Wert wurde erfolgreich ermittelt.
 */
EXTERN_DECL int __stdcall DFCComGetTime(int iComNum, int iBusNum, unsigned char *pucDateTime);

// ########################################################
/** 
 * @brief Direktes schreiben einer Textnachricht.
 *
 * Der gesendete Text wird fuer eine bestimmbare Zeit und mit einer
 * eventuell akustischen Signalisierung auf der Anzeige ausgegeben. Die
 * Ausgabe ist vom Geraetezustand abhaengig.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param ucDelayTime Anzeigedauer. (ucDelayTime * 250ms). 255->nicht automatisch wegblenden.
 * @param ucAction Ist fuer eingriffe in die Bedienung vorgesehen.
 * @param ucAudio Meldung aktustisch untermalen.
 * @param szMessage Zeiger auf den anzuzeigenden Text.
 * @param iLength Laenge des anzuzeigenden Textes.
 *
 * @return Ergebnis der Textsendung an Geraet.
 * @retval 0 Text konnte nicht geschrieben werden.
 * @retval 1 Text wurde erfolgreich geschrieben.
 */
EXTERN_DECL int __stdcall DFCComSendMessage(int iComNum, int iBusNum, unsigned char ucDelayTime, unsigned char ucAction, unsigned char ucAudio, char *szMessage, int iLength);

// ########################################################
/** 
 * @brief Indirektes schreiben einer Textnachricht.
 *
 * Der gesendete Text wird in einem eigens dafuer vorgesehenen Speicherbereich 
 * abgelegt und kann ueber die Pfeiltasten in den "nicht PZE-Moden" erreicht werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param szMessage Zeiger auf den anzuzeigenden Text.
 * @param iLength Laenge des anzuzeigenden Textes.
 *
 * @return Ergebnis der Textsendung an Geraet.
 * @retval 0 Text konnte nicht geschrieben werden.
 * @retval 1 Text wurde erfolgreich geschrieben.
 */
EXTERN_DECL int __stdcall DFCComSendInfotext(int iComNum, int iBusNum, char *szMessage,int iLength);

// ########################################################
/** 
 * @brief Die Seriennummer aus dem Geraet lesen.
 *
 * Gibt die gelesene Seriennummer als Ganzzahl ueber Parameter zurueck.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 * @param piSerialNum Zeiger auf Variable welche die Seriennummer erhaelt.
 *
 * @return Ergebnis der Werteermittlung.
 * @retval 0 Wert konnte nicht ermittelt werden und Fehlercode wurde ueber piError abgelegt.
 * @retval 1 Seriennummer wurde ueber piSerialNum in Variable uebergeben.
 */
EXTERN_DECL int __stdcall DFCGetSeriennummer(int iComNum, int iBusNum, int *piError, int *piSerialNum);


// ########################################################
/** 
 * @brief Schaltet die Logfunktionalitaet der DLL ein.
 *
 * Loggen von Kommunikationsablaeufen in die angegebene Datei.
 *
 * @param szFileName Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
 *
 * @return Ergebnis der Dateioeffnung.
 * @retval 0 Datei konnte nicht erstellt oder geoeffnet werden.
 * @retval 1 Logfunktionalitaet wurde erfolgreich eingeschaltet.
 *
 * @warning Ab der DLL - Version 2.0.3 wird diese Funktion nicht mehr unterstuetzt.
 * @warning Verwenden Sie bitte statt dessen die Funktionen DFCSetLogFileName() / DFCSetLogOn().
 */
EXTERN_DECL int __stdcall DFCLogOn(char *szFileName);

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Schaltet die Logfunktionalitaet des Kanals ein.
 *
 * Loggen von Kommunikationsablaeufen in die angegebene Datei.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.3 enthalten.
 */
EXTERN_DECL void __stdcall DFCSetLogOn(int iComNum);

#endif

// ########################################################
/** 
 * @brief Schaltet die Logfunktionalitaet der DLL aus.
 *
 * Abschalten des Logvorganges.
 *
 * @return Keine.
 *
 * @warning Ab der DLL - Version 2.0.3 wird diese Funktion nicht mehr unterstuetzt.
 * @warning Verwenden Sie bitte statt dessen die Funktion DFCSetLogOff().
 */
EXTERN_DECL void __stdcall DFCLogOff(void);

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Schaltet die Logfunktionalitaet des Kanals aus.
 *
 * Abschalten des Logvorganges.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.3 enthalten.
 */
EXTERN_DECL void __stdcall DFCSetLogOff(int iComNum);

#endif

#if (COM_VERSION >= 0x02000000)

// ########################################################
/** 
 * @brief Setzen einer Rueckruffunktion.
 *
 * Die Rueckruffunktion wird waehrend des Kommunikationsablaufs fortwaehrend 
 * aufgerufen und bietet die Moeglichkeit eines fruehzeitigen abbruches des selbigen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param CBFunction Zeiger auf die Rueckruffunktion oder 0 fuer abschalten.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.0 enthalten.
 */
EXTERN_DECL void __stdcall DFCSetCallBack(int iComNum, int (*CBFunction)());

#endif

#if (COM_VERSION >= 0x02000300)

// ########################################################
/** 
 * @brief Setzen des Dateinamen fuer die Logfunktionalitaet des Kanals.
 *
 * Bekanntgeben der Datei die fuer das Loggen verwendet wird.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param szLogFileName Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.3 enthalten.
 */
EXTERN_DECL void __stdcall DFCSetLogFileName(int iComNum, const unsigned char *szLogFileName);

// ########################################################
/** 
 * @brief Aufloesen einer Fehlernummer in seine lesbare Bedeutung.
 *
 * Die von den meisten Funktionen ueber den Parameter piError zurueckgegebene
 * Fehlernummer kann mit dieser Funktion in seine lesbare Bedeutung umgewandelt werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iError Fehlernummer.
 * @param iLanguage Angabe der zu verwendenden Sprache.
 * @param szText Zeiger auf die Zeichenfolge, welche den Fehlertext erhaelt.
 * @param iLength Laenge in Anzahl Byte des Speicherbereichs auf den szText verweist.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.3 enthalten.
 */
EXTERN_DECL void __stdcall DFCGetErrorText(int iComNum, int iError, int iLanguage, char *szText, int iLength);

#endif

#if (COM_VERSION >= 0x02000400)

// ########################################################
/** 
 * @brief Setzten des Wertes einer Globalen Variablen.
 *
 * Der Wert einer Globalen Variablen kann mit Hilfe dieser Funktion vom PC aus
 * abgeaendert werden. Diese muss natuerlich im verwendeten Geraetesetup definiert sein.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param pucVar Zeiger auf Byte welches die logische Variablennummer enthaelt, oder
 *               auf eine Zeichenfolge, welche den Variablennamen darstellt.
 * @param iType Gibt an ob pucVar auf eine Variablennummer oder auf einen Variablennamen verweist.
 * @param pucValue Zeiger auf den Wert der gesetzt werden soll.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Wertuebernahme im Geraet.
 * @retval 0 Wert konnte nicht uebernommen werden.
 * @retval 1 Wert wurde erfolgreich uebernommen.
 *
 * @version Ab DLL - Version 2.0.4 enthalten.
 */
EXTERN_DECL int __stdcall DFCSetGlobVar(int iComNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int *piError);

// ########################################################
/** 
 * @brief Lesen des Wertes einer Globalen Variablen.
 *
 * Der Wert einer Globalen Variablen kann mit Hilfe dieser Funktion vom PC aus
 * gelesen werden. Diese muss natuerlich im verwendeten Geraetesetup definiert sein.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param pucVar Zeiger auf Byte welches die logische Variablennummer enthaelt, oder
 *               auf eine Zeichenfolge, welche den Variablennamen darstellt.
 * @param iType Gibt an ob pucVar auf eine Variablennummer oder auf einen Variablennamen verweist.
 * @param pucValue Zeiger auf ermittelte nullterminierte Zeichenfolge.
 * @param iValueLength Laenge in Anzahl Byte des Speicherbereichs auf den pucValue verweist.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Wertermittlung aus Geraet.
 * @retval 0 Wert konnte nicht ermittelt werden.
 * @retval 1 Wert wurde erfolgreich ermittelt.
 *
 * @version Ab DLL - Version 2.0.4 enthalten.
 */
EXTERN_DECL int __stdcall DFCGetGlobVar(int iComNum, int iBusNum, unsigned char *pucVar, int iType, unsigned char *pucValue, int iValueLength, int *piError);

#endif

#if (COM_VERSION >= 0x02000500)

// ########################################################
/** 
 * @brief Schliessen eines angeschlossenen Relais.
 *
 * Internen digitalen Ausgang oder externen digitalen Ausgang (z.B. IOModule)
 * fuer eine bestimmbare Zeit schalten.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param iNum Gibt die Nummer des zu schaltenden Relais an.
 * @param iTime Gibt die Zeit in Sekunden an, in welcher das Relais geschaltet bleibt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCCloseRelay(int iComNum, int iBusNum, int iNum, int iTime, int *piError);

// ########################################################
/** 
 * @brief Abfragen von aktuellem Zustand und verbleibender Zeit.
 *
 * Der aktuelle Zustand und die verbleibende Zeit bei geschlossenem Relais abfragen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param iNum Gibt die Nummer des zu schaltenden Relais an.
 * @param piState Zeiger auf eine Variable, welche den aktuellen Schaltzustand des Ausgangs erhaelt.
 * @param piTime Zeiger auf die Variable, welche die verbleibende Zeit in Sekunden erhaelt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCGetRelayState(int iComNum, int iBusNum, int iNum, int *piState, int *piTime, int *piError);

// ########################################################
/** 
 * @brief Oeffnen eines Relais.
 *
 * Internen digitalen Ausgang oder externen digitalen Ausgang (z.B. IOModule) schalten.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param iNum Gibt die Nummer des zu schaltenden Relais an.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCOpenRelay(int iComNum, int iBusNum, int iNum, int *piError);

// ########################################################
/** 
 * @brief Abfrage der zuletzt mit DFCCheckDevice() benoetigten Versuche.
 *
 * Die Anzahl der benoetigten Versuche koennen fuer eventuelle Statistikzwecke 
 * verwendet werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Anzahl der benoetigten Versuche bis Geraet als verfuegbar galt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCGetDevicePollRetry(int iComNum);

#endif


// ########################################################
// ############## FUNKTIONEN FUER SETUP ###################
// ########################################################

// ########################################################
/** 
 * @brief Uebertragen einer Setupdatei.
 *
 * Es wird versucht die angegebene Setupdatei in das Geraet zu uebertragen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param szFileName Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 */
EXTERN_DECL int __stdcall DFCSetupLaden(int iComNum,int iBusNum, char *szFileName, int *piError);

#if (COM_VERSION >= 0x02000000)

// ########################################################
/** 
 * @brief Setupdaten aus Geraet lesen.
 *
 * Es wird versucht die Setupdaten aus dem Geraet in die angegebene Setupdatei 
 * zu uebertragen. 
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param szFileName Zeiger auf die Zeichenfolge, welche den Pfad und/oder Dateinamen enthaelt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.0 enthalten.
 */
EXTERN_DECL int __stdcall DFCDownload(int iComNum,int iBusNum, char *szFileName, int *piError);

#endif


// ########################################################
// ############## FUNKTIONEN FUER DATEN ###################
// ########################################################

// ########################################################
/** 
 * @brief Datenzeiger ruecksetzten.
 *
 * Die Datenzeiger im Geraet werden rueckgesetzt. Die im Geraet eventuell enthaltenen  
 * Datensaetze werden dadurch geloescht. 
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.0 enthalten.
 */
EXTERN_DECL int __stdcall DFCComClearData(int iComNum, int iBusNum);

// ########################################################
/** 
 * @brief Daten aus Geraet lesen.
 *
 * Es wird versucht die Daten aus dem Geraet in die DLL zu uebertragen. 
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @note Ab DLL - Version 2.0.0 satzweise lesen.
 */
EXTERN_DECL int __stdcall DFCComCollectData(int iComNum, int iBusNum, int *piError);

// ########################################################
/** 
 * @brief Datensatz aus DLL / Geraet lesen.
 *
 * Es wird versucht einen Datensatz aus der DLL / oder Geraet an den Aufrufer zu  
 * uebergeben. Die Busnummer ist durch vorherigen Aufruf der Funktion DFCComCollectData()
 * bekannt.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param pucBuf Zeiger auf den Puffer, welcher die Daten eines Datensatzes erhaelt.
 * @param piLength Zeiger auf Variable welche die Datensatzlaenge erhaelt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @note Ab DLL - Version 2.0.0 satzweise lesen.
 */
EXTERN_DECL int __stdcall DFCComGetDatensatz(int iComNum, unsigned char *pucBuf, int *piLength, int *piError);

#if (COM_VERSION >= 1)

// ########################################################
/** 
 * @brief Tabellenbeschreibungen fuer Datensaetze aus Geraet lesen.
 *
 * Es wird versucht die Tabellenbeschreibungen aus dem Geraet zu lesen. Hiernach koennen   
 * mit den Funktionen DFCDatBCnt(), DFCDatBDatensatz() und DFCDatBFeld() auf den 
 * ermittelten Daten gearbeitet werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCLoadDatensatzbeschreibung(int iComNum, int iBusNum, int *piError);

// ########################################################
/** 
 * @brief Anzahl der Tabellenbeschreibungen fuer Datensaetze ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Daten, wird 
 * die Anzahl der definierten Tabellen zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Anzahl der mit DFCLoadDatensatzbeschreibung() gelesenen Tabellendefinitionen.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCDatBCnt(int iComNum);

// ########################################################
/** 
 * @brief Grunddaten einer Tabellenbeschreibungen fuer Datensaetze ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Daten, werden 
 * die Grunddaten zu einer Tabellen zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iDataNum Tabellennummer im Bereich von (0 - (DFCDatBCnt() - 1)).
 * @param szName Zeiger auf Speicherbereich fuer Tabellenname. (Mindestens 17 Zeichen).
 * @param piFieldCount Zeiger auf Variable, welche die Feldanzahl der Tabelle erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCDatBDatensatz(int iComNum, int iDataNum, unsigned char *szName, int *piFieldCount);

// ########################################################
/** 
 * @brief Grunddaten eines Tabellenfeldes fuer Datensaetze ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadDatensatzbeschreibung() gelesenen Daten, werden 
 * die Grunddaten zu einem Tabellenfeld zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iDataNum Tabellennummer im Bereich von (0 - (DFCDatBCnt() - 1)).
 * @param iField Feldnummer.
 * @param szName Zeiger auf Speicherbereich fuer Feldname. (Mindestens 17 Zeichen).
 * @param piType Zeiger auf Variable, welche den Feldtyp erhaelt.
 * @param piLength Zeiger auf Variable, welche die definierte Laenge des Feldes erhaelt,
 *                 in dieser ist auch das Nullterminierungszeichen enthalten.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCDatBFeld(int iComNum, int iDataNum, int iField, unsigned char *szName, int *piType, int *piLength);

#endif


// ########################################################
// ############## FUNKTIONEN FUER LISTEN ##################
// ########################################################

// ########################################################
/** 
 * @brief Setzen des Listenbuffers fuer eine Liste.
 *
 * Die Listen muessen ihrer Reichenfolge nach (0 .. n) in den DLL internen 
 * Buffer uebertragen werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iListNum Tabellennummer.
 * @param iLineCount Anzahl der Zeilen, Datensaetze im Block.
 * @param iListSize Anzahl Byte im Block. Brechnet sich aus (Anzahl Datensaetzte * Datensatzlaenge).
 * @param pucBuf Zeiger auf Bytearray, welches die Daten enthaelt.
 * @param iListMemSizeNum Kann immer mit 0 uebergeben werden. (Kompatibel mit Timeboy)
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 */
EXTERN_DECL int __stdcall DFCMakeListe(int iComNum, int iListNum, int iLineCount, int iListSize, unsigned char *pucBuf, int iListMemSizeNum);

// ########################################################
/** 
 * @brief Uebertragen der Listendaten zum Geraet.
 *
 * Es wird versucht, die zuvor mit DFCMakeListe() importierten Listendaten in 
 * das Geraet zu uebertragen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 */
EXTERN_DECL int __stdcall DFCLoadListen(int iComNum, int iBusNum, int *piError);

// ########################################################
/** 
 * @brief Buffer fuer Listendaten loeschen.
 *
 * Loescht in der DLL den Buffer fuer Listendaten. Die zuvor schon importierten 
 * Listen waehren nach Ausfuehrung wiederholt zu importieren.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Keine.
 */
EXTERN_DECL void __stdcall DFCClrListenBuffer(int iComNum);

#if (COM_VERSION >= 1)

// ########################################################
/** 
 * @brief Tabellenbeschreibungen fuer Listen aus Geraet lesen.
 *
 * Es wird versucht die Tabellenbeschreibungen aus dem Geraet zu lesen. Hiernach koennen   
 * mit den Funktionen DFCListBCnt(), DFCListBDatensatz() und DFCListBFeld() auf den ermittelten 
 * Daten gearbeitet werden.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCLoadListenbeschreibung(int iComNum, int iBusNum, int *piError);

// ########################################################
/** 
 * @brief Anzahl der Tabellenbeschreibungen fuer Listen ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, wird 
 * die Anzahl der definierten Tabellen zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 *
 * @return Anzahl der mit DFCLoadListenbeschreibung() gelesenen Tabellendefinitionen.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCListBCnt(int iComNum);

// ########################################################
/** 
 * @brief Grunddaten einer Tabellenbeschreibungen fuer Listen ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, werden 
 * die Grunddaten zu einer Tabellen zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iDataNum Tabellennummer im Bereich von (0 - (DFCListBCnt() - 1)).
 * @param szName Zeiger auf Speicherbereich fuer Tabellenname. (Mindestens 17 Zeichen).
 * @param piFieldCount Zeiger auf Variable, welche die Feldanzahl der Tabelle erhaelt.
 * @param piVar Zeiger auf Variable, welche immer eine 0 erhaelt. (Kompatible zu Timeboy)
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCListBDatensatz(int iComNum, int iDataNum, unsigned char *szName, int *piFieldCount, int *piVar);

// ########################################################
/** 
 * @brief Grunddaten eines Tabellenfeldes fuer Listen ermitteln.
 *
 * Auf Grundlage der zuvor mit DFCLoadListenbeschreibung() gelesenen Daten, werden 
 * die Grunddaten zu einem Tabellenfeld zurueckgegeben.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iDataNum Tabellennummer im Bereich von (0 - (DFCListBCnt() - 1)).
 * @param iField Feldnummer.
 * @param szName Zeiger auf Speicherbereich fuer Feldname. (Mindestens 17 Zeichen).
 * @param piType Zeiger auf Variable, welche den Feldtyp erhaelt.
 * @param piLength Zeiger auf Variable, welche die definierte Laenge des Feldes erhaelt,
 *                 in dieser ist auch das Nullterminierungszeichen enthalten.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden. (Tabelle nicht definiert!)
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 1.1.i enthalten.
 */
EXTERN_DECL int __stdcall DFCListBFeld(int iComNum, int iDataNum, int iField, unsigned char *szName, int *piType, int *piLength);

#endif


// ########################################################
// ####### FUNKTIONEN FUER ZUTRITTSKONTROLLLISTEN #########
// ########################################################

#if (COM_VERSION >= 0x02000500)

// ########################################################
/** 
 * @brief Setzen des Listenbuffers fuer eine Liste.
 *
 * Die Listen werden zusaetzlich intern komprimiert.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iListNum Tabellennummer.
 * @param iLineCount Anzahl der Zeilen, Datensaetze im Block.
 * @param iListSize Anzahl Byte im Block. Brechnet sich aus (Anzahl Datensaetzte * Datensatzlaenge).
 * @param pucBuf Zeiger auf Bytearray, welches die Daten enthaelt.
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCMakeEntranceList(int iComNum, int iListNum, int iLineCount, int iListSize, unsigned char *pucBuf, int *piError);

// ########################################################
/** 
 * @brief Uebertragen der Listendaten zum Geraet.
 *
 * Es wird versucht, die zuvor mit DFCMakeEntranceListe() importierten Listendaten in 
 * das Geraet zu uebertragen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Busnummer des anzusprechenden Geraetes.
 * @param iListNum Tabellennummer der zu uebertragenden Liste. (-1 uebertraegt alle importierten)
 * @param piError Zeiger auf Variable die nach Fehler den Fehlercode erhaelt.
 *
 * @return Ergebnis der Ausfuehrung.
 * @retval 0 Funktion konnte nicht ausgefuehrt werden.
 * @retval 1 Funktion wurde erfolgreich ausgefuehrt.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL int __stdcall DFCLoadEntranceList(int iComNum, int iBusNum, int iListNum, int *piError);

// ########################################################
/** 
 * @brief Buffer fuer Listendaten loeschen.
 *
 * Loescht in der DLL den Buffer fuer Listendaten.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iListNum Listennummer (0 - 3). Bei Angabe von -1 werden alle geloescht.
 *
 * @return Keine.
 *
 * @version Ab DLL - Version 2.0.5 enthalten.
 */
EXTERN_DECL void __stdcall DFCClearEntranceListBuffer(int iComNum, int iListNum);

#endif


// ########################################################
// ############ FUNKTIONEN FUER ERWEITERUNG ###############
// ########################################################

#if (COM_VERSION >= 0x02000900)

// ########################################################
/** 
 * @brief Oeffnet die Schnittstelle fuer die Kommunikation mit PZE Master IV.
 *
 * Initialisiert den Kanal. 
 * Je nach angegebenem Typ wird ein anderes Initialisierungsverfahren verwendet 
 * um die entsprechend nachfolgenden Funktionsaufrufe der exportierten Funktionen 
 * richtig umzusetzen.
 *
 * @param iComNum Kommunikationskanalnummer.
 * @param iBusNum Die von der DLL zu verwendende Busnummer. (Multimasteransatz)
 * @param iType Der zu verwendende Schnittstellentyp.
 *              1 -> RS232 direkt oder ber Umsetzter (RS232 <-> RS485)
 *              2 -> Funk ber den Umsetzer (RS232 <-> BIM2) ohne eigenen Kontroller.
 *              3 -> TCP / IP - Verbindung.
 * @param szComString Je nach Schnittstellentyp:
 *                    1,2 -> Zeiger auf Zeichenkette fr die Schnittstelle. "COM1" .. "COM4".
 *                    3 -> Zeiger auf Zeichenkette mit IP - Adresse oder Hostname.
 * @param iComValue Je nach Schnittstellentyp:
 *                  1,2 -> bertragungsrate der Schnittstelle. (Standard ist: 38400)
 *                  3 -> Portnummer des TCP / IP Anschlusses.
 * @param iComTimeOut Je nach Schnittstellentyp:
 *                    1,2 ->Timeout fr Lesen von Schnittstelle (Standard ist: 150)
 *                    3 -> Timeout fr Lesen von Socket (Standard ist: 3000)
 *
 * @return Kommunikationskanal oeffnen.
 * @retval 1 Schnittstelle erfolgreich geoeffnet.
 * @retval 0 Fehler bei Initialisierung.
 */
EXTERN_DECL int __stdcall DFCComOpenIV(int iComNum, int iBusNum, int iType, char *szComString, int iComValue, int iComTimeOut);

#endif

// ########################################################
// ############# ** PRIVATE FUNKTIONEN ** #################
// ########################################################

// Funktionen sind nicht dokumentiert.
// Sie werden von AESetup oder Talk verwendet.
// Unterliegen evntl. Aenderungen.

#if (USE_PRIVATE)

// ########################################################
EXTERN_DECL int __stdcall DFCComUseHandle(int iComNum, HANDLE hCom);

// ########################################################
EXTERN_DECL int __stdcall DFCComReleaseHandle(int iComNum);

#if (AE3COM_EXPORT && (COM_VERSION >= 0x02000000))

// ########################################################
extern "C" HRESULT PASCAL EXPORT CALLBACK DllGetVersion(DLLVERSIONINFO *pdvi);

#endif

#if (COM_VERSION >= 0x02000200)

// ########################################################
EXTERN_DECL int __stdcall DFCComVersion(int iComNum, int iBusNum, int *piError);

#endif

#if (COM_VERSION >= 0x02000300)

// ########################################################
EXTERN_DECL HANDLE __stdcall DFCGetComPort(int iComNum);

// ########################################################
EXTERN_DECL int __stdcall DFCSetComPort(int iComNum, HANDLE hComPort, int bPortIsSocket, int iBaudRate, int iTimeOut);

#endif

#if (COM_VERSION >= 0x02000400)

// ########################################################
EXTERN_DECL int __stdcall DFCCheckData(int iComNum, int iBusNum, int *piError);

#endif

#endif // USE_PRIVATE

#endif
